@@ -19,6 +19,7 @@
 #include <solid/device.h>
 #include <solid/genericinterface.h>
 
+#include <QLocale>
 #include <QMimeDatabase>
 
 using namespace Solid::Backends::UDisks2;
@@ -65,13 +66,34 @@ static QString formatByteSize(double size)
     return s;
 }
 
+static QString concatBlockDeviceDescription(const QString &name, qulonglong size, bool isExternal)
+{
+    QString description;
+    if (size > 0) {
+        const QString sizeStr = formatByteSize(size);
+        if (isExternal) {
+            description = QObject::tr("%1 External Drive (%2)", "%1 is the size, %2 is the block device name e.g. sda, sda1").arg(sizeStr, name);
+        } else {
+            description = QObject::tr("%1 Internal Drive (%2)", "%1 is the size, %2 is the block device name e.g. sda, sda1").arg(sizeStr, name);
+        }
+    } else {
+        if (isExternal) {
+            description = QObject::tr("External Drive (%1)", "%1 is the block device name e.g. sda, sda1").arg(name);
+        } else {
+            description = QObject::tr("Internal Drive (%1)", "%1 is the block device name e.g. sda, sda1").arg(name);
+        }
+    }
+
+    return description;
+}
+
 Device::Device(const QString &udi)
     : Solid::Ifaces::Device()
     , m_backend(DeviceBackend::backendForUDI(udi))
 {
     if (m_backend) {
-        connect(m_backend, SIGNAL(changed()), this, SIGNAL(changed()));
-        connect(m_backend, SIGNAL(propertyChanged(QMap<QString, int>)), this, SIGNAL(propertyChanged(QMap<QString, int>)));
+        connect(m_backend, &DeviceBackend::changed, this, &Device::changed);
+        connect(m_backend, &DeviceBackend::propertyChanged, this, &Device::propertyChanged);
     } else {
         qCDebug(UDISKS2) << "Created invalid Device for udi" << udi;
     }
@@ -350,20 +372,9 @@ QString Device::storageDescription() const
     const bool drive_is_removable = storageDrive.isRemovable();
 
     if (drive_type == Solid::StorageDrive::HardDisk && !drive_is_removable) {
-        QString size_str = formatByteSize(storageDrive.size());
-        if (storageDrive.size() > 0) {
-            if (drive_is_hotpluggable) {
-                description = tr("%1 External Hard Drive", "%1 is the size").arg(size_str);
-            } else {
-                description = tr("%1 Hard Drive", "%1 is the size").arg(size_str);
-            }
-        } else {
-            if (drive_is_hotpluggable) {
-                description = tr("External Hard Drive");
-            } else {
-                description = tr("Hard Drive");
-            }
-        }
+        QString devName = storageDrive.device();
+        devName.remove(QLatin1String("/dev/"));
+        description = concatBlockDeviceDescription(devName, storageDrive.size(), drive_is_hotpluggable);
 
         return description;
     }
@@ -549,9 +560,10 @@ QString Device::volumeDescription() const
     }
 
     const bool drive_is_removable = storageDrive.isRemovable();
-    const bool drive_is_hotpluggable = storageDrive.isHotpluggable();
 
     QString size_str = formatByteSize(storageVolume.size());
+    QString volumeName = storageVolume.device();
+    volumeName.remove(QLatin1String("/dev/"));
     if (isEncryptedContainer()) {
         if (storageVolume.size() > 0) {
             description = tr("%1 Encrypted Drive", "%1 is the size").arg(size_str);
@@ -559,26 +571,22 @@ QString Device::volumeDescription() const
             description = tr("Encrypted Drive");
         }
     } else if (drive_type == Solid::StorageDrive::HardDisk && !drive_is_removable) {
-        if (storageVolume.size() > 0) {
-            if (drive_is_hotpluggable) {
-                description = tr("%1 External Hard Drive", "%1 is the size").arg(size_str);
-            } else {
-                description = tr("%1 Hard Drive", "%1 is the size").arg(size_str);
-            }
-        } else {
-            if (drive_is_hotpluggable) {
-                description = tr("External Hard Drive");
-            } else {
-                description = tr("Hard Drive");
-            }
-        }
+        description = concatBlockDeviceDescription(volumeName, storageVolume.size(), storageDrive.isHotpluggable());
     } else if (drive_type == Solid::StorageDrive::Floppy) {
         description = tr("Floppy Disk");
     } else {
         if (drive_is_removable) {
-            description = tr("%1 Removable Media", "%1 is the size").arg(size_str);
+            if (storageVolume.size() > 0) {
+                description = tr("%1 Removable Media", "%1 is the size").arg(size_str);
+            } else {
+                description = tr("Removable Media");
+            }
         } else {
-            description = tr("%1 Media", "%1 is the size").arg(size_str);
+            if (storageVolume.size() > 0) {
+                description = tr("%1 Media", "%1 is the size").arg(size_str);
+            } else {
+                description = tr("Storage Media");
+            }
         }
     }
 
@@ -746,6 +754,10 @@ QString Device::errorToString(const QString &error) const
         return tr("The operation would wake up a disk that is in a deep-sleep state");
     } else if (error == UD2_ERROR_ALREADY_CANCELLED) {
         return tr("The operation has already been canceled");
+    } else if (error == UD2_ERROR_NOT_AUTHORIZED_CAN_OBTAIN) {
+        return tr("Cannot request authentication for this action. The PolicyKit authentication system appears to be not available.");
+    } else if (error == UD2_ERROR_NOT_AUTHORIZED_DISMISSED) {
+        return tr("The authentication prompt was canceled");
     } else {
         return tr("An unspecified error has occurred");
     }
@@ -763,6 +775,8 @@ Solid::ErrorType Device::errorToSolidError(const QString &error) const
         return Solid::InvalidOption;
     } else if (error == UD2_ERROR_MISSING_DRIVER) {
         return Solid::MissingDriver;
+    } else if (error == UD2_ERROR_NOT_AUTHORIZED_DISMISSED) {
+        return Solid::UserCanceled;
     } else {
         return Solid::UnauthorizedOperation;
     }
@@ -869,3 +883,5 @@ QString Device::drivePath() const
 {
     return prop("Drive").value<QDBusObjectPath>().path();
 }
+
+#include "moc_udisksdevice.cpp"
